@@ -462,7 +462,13 @@ symbol_class_set (symbol *sym, symbol_class class, location loc, bool declaring)
       if (declaring)
         {
           if (s->status == declared)
-            complain (&loc, Wother, _("symbol %s redeclared"), sym->tag);
+            {
+              unsigned i = 0;
+              complain (&loc, Wother, _("symbol %s redeclared"), sym->tag);
+              i += SUB_INDENT;
+              complain_indent (&sym->location, Wother, &i,
+                               _("previous declaration"));
+            }
           else
             s->status = declared;
         }
@@ -489,7 +495,7 @@ symbol_user_token_number_set (symbol *sym, int user_token_number, location loc)
     {
       *user_token_numberp = user_token_number;
       /* User defined $end token? */
-      if (user_token_number == 0)
+      if (user_token_number == 0 && !endtoken)
         {
           endtoken = sym->content->symbol;
           /* It is always mapped to 0, so it was already counted in
@@ -773,16 +779,16 @@ hash_semantic_type_hasher (void const *m, size_t tablesize)
 void
 symbols_new (void)
 {
-  symbol_table = hash_initialize (HT_INITIAL_CAPACITY,
-                                  NULL,
-                                  hash_symbol_hasher,
-                                  hash_symbol_comparator,
-                                  symbol_free);
-  semantic_type_table = hash_initialize (HT_INITIAL_CAPACITY,
-                                         NULL,
-                                         hash_semantic_type_hasher,
-                                         hash_semantic_type_comparator,
-                                         free);
+  symbol_table = hash_xinitialize (HT_INITIAL_CAPACITY,
+                                   NULL,
+                                   hash_symbol_hasher,
+                                   hash_symbol_comparator,
+                                   symbol_free);
+  semantic_type_table = hash_xinitialize (HT_INITIAL_CAPACITY,
+                                          NULL,
+                                          hash_semantic_type_hasher,
+                                          hash_semantic_type_comparator,
+                                          free);
 }
 
 
